from Globals import DTMLFile, InitializeClass, package_home
from OFS.SimpleItem import Item
from Acquisition import Implicit
from Persistence import Persistent
from App.FindHomes import INSTANCE_HOME, SOFTWARE_HOME
from os.path import join, exists, split
from os import environ

class SuperLogger(Item, Implicit, Persistent):
    """ """
    id = 'SuperLogger'
    title = meta_type = 'SuperLogger'
    manage_options = ({'label': 'Configure', 'action':'configureSuperLoggerForm'},{'label': 'Install', 'action':'installSuperLoggerForm'})
    modifyloc = join(INSTANCE_HOME, 'z2.py')
    try:
        # ZOPE_HOME only exists in 2.6
        ZOPE_HOME = environ['ZOPE_HOME']
    except:
        # so otherwise, let's figure it out ourselves
        home = SOFTWARE_HOME
        home = split(home)[0]
        home = split(home)[0]
        ZOPE_HOME = home

    replaceloc = join(ZOPE_HOME, 'ZServer', 'medusa')

    configureSuperLoggerForm = DTMLFile('dtml/configureForm', globals())
    installSuperLoggerForm = DTMLFile('dtml/installForm', globals())
    installResultsForm = DTMLFile('dtml/installResultsForm', globals())

    def __init__(self):
        """ """
        self.configureSuperLogger()

    def configureSuperLogger(self, virtualhost = 0, fileformat = 1, whitelist = 0, rotational = 0, frequency = 'monthly', hosts = [], REQUEST=None):
        """ """
        self.virtualhost = int(virtualhost)
        self.fileformat = int(fileformat)
        self.whitelist = int(whitelist)
        self.rotational = int(rotational)
        self.frequency = frequency
        self.hosts = hosts
        if REQUEST is not None:
            return self.configureSuperLoggerForm(self, REQUEST)

    def installSuperLogger(self, replaceloc, modifyloc, replace = 0, modify = 0, REQUEST = None):
        """ Automatic Install of SuperLogger """
        replaced = 0
        replacemessage = ''
        if replace:
            if exists(join(replaceloc, 'logger.py.bak')) or exists(join(replaceloc, 'http_server.py.bak')):
                replacemessage = 'logger.py.bak and/or http_server.py.bak files exist already'
            else:
                try:
                    replacelogfiles(replaceloc)
                    replaced = 1
                    replacemessage = 'Originals backed up to logger.py.bak and http_server.py.bak'
                except OSError, details:
                    replacemessage = details

        modified = 0
        modifymessage = ''
        if modify:
            try:
                modifyZ2(modifyloc)
                modified = 1
            except IndexError, details:
                modifymessage = details
            except OSError, details:
                modifymessage = details

        if REQUEST:
            return self.installResultsForm(self, REQUEST,
                                           replaced = replaced,
                                           replacemessage = replacemessage,
                                           modified = modified,
                                           modifymessage = modifymessage)

    def get_variables(self):
        # used in the start up script to get the settings for the logger
        v = {}
        v['virtualhost'] = self.virtualhost
        v['fileformat'] = self.fileformat
        v['whitelist'] = self.whitelist
        v['hosts'] = self.get_hosts()
        v['rotational'] = self.rotational
        v['frequency'] = self.frequency
        return v

    def get_hosts(self):
        """ """
        from Acquisition import aq_base
        hosts = []
        if self.whitelist == 1:
            # get the hosts list from a VirtualHostMonster sitting in the root
            vhms = self.getPhysicalRoot().objectValues('Virtual Host Monster')
            # only do this if there is one and only one VHM in the root
            if len(vhms) == 1:
                hosts = vhms[0].lines
                hosts = map(lambda x: x.split('/')[0], hosts)
        if self.whitelist == 2:
            # get the hosts list from the internal list
            hosts = self.hosts
        return hosts
    
InitializeClass(SuperLogger)


def modifyZ2(filename):
    def find(reference, start = 0):
        ret = -1
        for i in range(start, len(z2)):
            f = z2[i].find(reference)
            if f != -1:
                # make sure the line isn't commented
                c = z2[i].find('#')
                if c == -1 or f < c:
                    ret = i
                    break
        return ret

    global z2
    file = open(filename, 'r')
    z2 = file.readlines()
    file.close()

    # make sure there is no reference to SuperLogger already
    if find('logger.SuperLogger') != -1:
        raise IndexError, 'SuperLogger is already mentioned in %s' % filename

    # find file_logger
    search = 'lg = logger.file_logger(LOG_PATH)'
    linef = find(search)
    if linef == -1:
        raise IndexError, 'No correct reference to file_logger(LOG_PATH) in %s' % filename
    line = z2[linef]
    f = line.find(search)
    space = ' ' * f
    tab = ' ' * 4
    z2[linef] = line[:f] + tab + line[f:]   # extra tab for file_logger cos it'll be inside an else
    changes = []
    changes.append("if MODULE == 'Zope':")
    changes.append(tab + 'try: Zope == Zope')
    changes.append(tab + 'except: import Zope')
    changes.append(tab + 'lg = logger.SuperLogger(LOG_PATH, Zope)')
    changes.append('else:')
    changes.reverse()
    for line in changes:
        z2.insert(linef, '%s%s\n' % (space, line))
    file = open(filename, 'w')
    file.writelines(z2)
    file.close()


def replacelogfiles(directory):
    from shutil import copyfile
    for file in ('logger.py', 'http_server.py'):
        filename = join(directory, file)
        # back up file
        copyfile(filename, '%s.bak' % filename)
        # copy over original
        installloc = join(package_home(globals()), 'install')
        copyfile(join(installloc, file), filename)
